@@ -30,6 +30,13 @@ class SingleImgImporter(object):
         else:
             # update existing one
             doc_url = self.core.call_success("doc_id_to_url", doc_id)
+            if self.core.call_success("doc_is_readonly_by_url", doc_url):
+                LOGGER.warning(
+                    "Doc %s is read-only, using a new one instead", doc_url
+                )
+                (doc_id, doc_url) = self.core.call_success(
+                    "storage_get_new_doc"
+                )
 
         nb_pages = self.core.call_success("doc_get_nb_pages_by_url", doc_url)
         if nb_pages is None:
@@ -100,7 +107,10 @@ class SingleImgImporterFactory(object):
         if file_ext in self.plugin.FILE_EXTENSIONS:
             return True
 
-    def make_importer(self, file_import, file_uri):
+    def get_required_data(self, file_uri):
+        return set()
+
+    def make_importer(self, file_import, file_uri, data):
         return SingleImgImporter(self, file_import, file_uri)
 
 
@@ -119,6 +129,7 @@ class Plugin(openpaperwork_core.PluginBase):
         "jpeg",
         "jpg",
         "png",
+        "tif",
         "tiff",
     ]
 
@@ -146,7 +157,7 @@ class Plugin(openpaperwork_core.PluginBase):
             },
             {
                 'interface': 'pillow',
-                'defaults': ['paperwork_backend.pillow.img'],
+                'defaults': ['openpaperwork_core.pillow.img'],
             },
             {
                 'interface': 'thread',
@@ -154,8 +165,8 @@ class Plugin(openpaperwork_core.PluginBase):
             },
         ]
 
-    def get_import_mime_type(self, out: list):
-        out += self.IMG_MIME_TYPES
+    def get_import_mime_types(self, out: set):
+        out.update(self.IMG_MIME_TYPES)
 
     def get_importer(self, out: list, file_import: FileImport):
         importer = DirectFileImporter(
