@@ -10,6 +10,8 @@ It stores data in 2 ways:
 import collections
 import gc
 import logging
+import os
+import pickle
 import sqlite3
 import threading
 import time
@@ -245,6 +247,9 @@ class UpdatableVectorizer(object):
             )
             doc_vector = list(doc_vector)
             doc_vector = doc_vector[0][0]
+            if doc_vector is None:
+                # may happen according to bug report #478
+                doc_vector = numpy.array([])
 
             to_drop_for_this_doc = [f for f in to_drop if f < len(doc_vector)]
             doc_vector = numpy.delete(doc_vector, to_drop_for_this_doc)
@@ -330,7 +335,7 @@ class Corpus(object):
             required_padding = (
                 vectorizer.last_feature_id + 1 - doc_vector.shape[0]
             )
-            assert(required_padding >= 0)
+            assert required_padding >= 0
             if required_padding > 0:
                 doc_vector = scipy.sparse.hstack([
                     scipy.sparse.csr_matrix(doc_vector),
@@ -392,7 +397,7 @@ class Corpus(object):
     def _add_doc_ids(max_doc_backlog, doc_weights, doc_ids):
         # Assumes doc_ids are in reverse order (most recent first)
         # Also assumes most recent documents are the most useful
-        assert(len(doc_ids) <= max_doc_backlog)
+        assert len(doc_ids) <= max_doc_backlog
         weigth = max_doc_backlog + 1
         for doc_id in doc_ids:
             doc_weights[doc_id] += weigth
@@ -533,7 +538,7 @@ class LabelGuesserTransaction(sync.BaseTransaction):
             return
 
         # prevent reload of classifiers during the transaction
-        assert(not self.lock_acquired)
+        assert not self.lock_acquired
         self.plugin.classifiers_cond.acquire()
         self.lock_acquired = True
 
@@ -673,12 +678,16 @@ class LabelGuesserTransaction(sync.BaseTransaction):
     def commit(self):
         try:
             LOGGER.info("Committing")
+
+            if self.nb_changes > 0 and os.path.exists(self.plugin.model_path):
+                os.unlink(self.plugin.model_path)
+
             self.core.call_success(
                 "mainloop_schedule", self.core.call_all,
                 "on_label_guesser_commit_start"
             )
             if self.nb_changes <= 0:
-                assert(self.cursor is None)
+                assert self.cursor is None
                 self.notify_done(ID)
                 self.core.call_success(
                     "mainloop_schedule", self.core.call_all,
@@ -747,6 +756,10 @@ class Plugin(openpaperwork_core.PluginBase):
 
         self.bayes_dir = None
         self.sql_url = None
+        # We pickle the trained model. But pickle and Gio don't work well
+        # together --> therefore we work directly with a path here instead of
+        # an URL and with `os` instead `self.core.call_xxx("fs_xxx")`
+        self.model_path = None
 
         SqliteNumpyArrayHandler.register()
 
@@ -817,7 +830,8 @@ class Plugin(openpaperwork_core.PluginBase):
 
     def _init(self):
         data_dir = self.core.call_success(
-            "data_dir_handler_get_individual_data_dir")
+            "data_dir_handler_get_individual_data_dir"
+        )
 
         if self.bayes_dir is None:  # may be set by tests
             self.bayes_dir = self.core.call_success(
@@ -829,6 +843,11 @@ class Plugin(openpaperwork_core.PluginBase):
         self.sql_url = self.core.call_success(
             "fs_join", self.bayes_dir, 'label_guesser.db'
         )
+        self.model_path = self.core.call_success(
+            "fs_unsafe", self.core.call_success(
+                "fs_join", self.bayes_dir, "model.pickle"
+            )
+        )
         LOGGER.info("Opening guesswork.label.sklearn database")
         self.sql = self.core.call_one(
             "sqlite_execute",
@@ -889,7 +908,7 @@ class Plugin(openpaperwork_core.PluginBase):
                     (
                         self.word_reductor, self.classifiers
                     ) = self._load_classifiers(
-                        cursor, self.vectorizer
+                        self.model_path, cursor, self.vectorizer
                     )
                 finally:
                     cursor.execute("ROLLBACK")
@@ -897,7 +916,15 @@ class Plugin(openpaperwork_core.PluginBase):
             finally:
                 self.classifiers_cond.notify_all()
 
-    def _load_classifiers(self, cursor, vectorizer):
+    def _load_classifiers(self, model_path, cursor, vectorizer):
+        if os.path.exists(model_path):
+            with open(model_path, "rb") as fd:
+                LOGGER.info("Pre-trained model available. Using it")
+                (reductor, classifiers) = pickle.load(fd)
+            return (reductor, classifiers)
+
+        LOGGER.info("Pre-trained model not available. Re-training one")
+
         # Jflesch> This is a very memory-intensive process. The Glib may try
         # to allocate memory before the GC runs and AFAIK the Glib
         # is not aware of Python GC, and so won't trigger it if required
@@ -990,6 +1017,8 @@ class Plugin(openpaperwork_core.PluginBase):
             # --> free as much memory as possible now
             gc.collect()
 
+            with open(model_path, "wb") as fd:
+                pickle.dump((reductor, classifiers), fd)
             return (reductor, classifiers)
         finally:
             self.core.call_all("on_progress", "label_classifiers", 1.0, msg)
@@ -1000,10 +1029,18 @@ class Plugin(openpaperwork_core.PluginBase):
         self.core.call_all("doc_get_text_by_url", doc_txt, doc_url)
         doc_txt = "\n\n".join(doc_txt).strip()
         if doc_txt == u"":
+            LOGGER.info("Can't guess. Text is empty")
             return
 
         vector = vectorizer.transform([doc_txt])
-        vector = vector.toarray()[0]
+        vector = vector.toarray()
+        if len(vector) <= 0:
+            LOGGER.info(
+                "Failed to vectorize text (text length=%d)",
+                len(doc_txt)
+            )
+            return
+        vector = vector[0]
         vector = reductor.reduce_features(vector)
 
         min_features = self.config.get("min_features")
@@ -1029,7 +1066,7 @@ class Plugin(openpaperwork_core.PluginBase):
 
     def _set_guessed_labels(self, doc_url):
         # self.classifiers_cond must locked
-        assert(self.classifiers is not None)
+        assert self.classifiers is not None
         labels = self._guess(
             self.vectorizer, self.word_reductor, self.classifiers, doc_url
         )
