@@ -40,6 +40,24 @@ class Plugin(openpaperwork_core.PluginBase):
             )
         )
         self.core.call_all("config_register", "workdir", setting)
+        self.core.call_all(
+            "config_add_observer", "workdir", self._on_work_dir_changed
+        )
+
+    def _on_work_dir_changed(self):
+        LOGGER.info("Work directory has changed")
+        self.core.call_all("on_storage_changed")
+
+    def storage_get_id(self):
+        """
+        Returns a string identifying the storage (work directory)
+        currently used.
+
+        Do not assume it's a valid directory path. For instance, this plugin
+        could be replaced by a MariaDB database someday, and so the string
+        would identify the database instead of a directory.
+        """
+        return self.core.call_success('config_get', 'workdir')
 
     def storage_get_all_docs(self, out: list, only_valid=True):
         """
@@ -47,14 +65,23 @@ class Plugin(openpaperwork_core.PluginBase):
         """
         workdir = self.core.call_success('config_get', 'workdir')
         if self.core.call_success('fs_exists', workdir) is None:
-            # we are not the plugin handling this work directory (?)
+            # we are not the plugin handling this storage (?)
             return
         LOGGER.info("Loading document list from %s", workdir)
         nb = 0
         for doc_url in self.core.call_success('fs_listdir', workdir):
+            if only_valid and not self.core.call_success("fs_isdir", doc_url):
+                LOGGER.warning(
+                    "Ignoring document '%s': not a directory", doc_url
+                )
+                continue
             if (
                     only_valid and
                     self.core.call_success("is_doc", doc_url) is None):
+                LOGGER.warning(
+                    "Ignoring document '%s': not recognized document type",
+                    doc_url
+                )
                 continue
             out.append((
                 self.core.call_success("fs_basename", doc_url), doc_url
@@ -63,7 +90,7 @@ class Plugin(openpaperwork_core.PluginBase):
         LOGGER.info("%d documents found in %s", nb, workdir)
 
     def doc_id_to_url(self, doc_id, existing=True):
-        assert(doc_id is not None)
+        assert doc_id is not None
         workdir = self.core.call_success('config_get', 'workdir')
         url = self.core.call_success("fs_join", workdir, doc_id)
         if existing and self.core.call_success("fs_isdir", url) is None:
@@ -112,7 +139,7 @@ class Plugin(openpaperwork_core.PluginBase):
         doc_url = self.doc_id_to_url(doc_id)
         if doc_url is None:
             return
-        self.core.call_all("fs_rm_rf", doc_url, trash=trash)
+        self.core.call_success("fs_rm_rf", doc_url, trash=trash)
 
     def stats_get(self, stats):
         LOGGER.info("Counting documents for statistics...")
@@ -131,7 +158,7 @@ class Plugin(openpaperwork_core.PluginBase):
             "All pages of document %s have been removed. Removing document",
             doc_url
         )
-        self.core.call_all("fs_rm_rf", doc_url)
+        self.core.call_success("fs_rm_rf", doc_url)
 
     def page_move_by_url(
                 self,
@@ -154,7 +181,7 @@ class Plugin(openpaperwork_core.PluginBase):
             "All pages of document %s have been removed. Removing document",
             source_doc_url
         )
-        self.core.call_all("fs_rm_rf", source_doc_url)
+        self.core.call_success("fs_rm_rf", source_doc_url)
 
     def doc_rename_by_url(self, src_doc_url, dst_doc_url):
         if not self.core.call_success("fs_isdir", src_doc_url):
@@ -166,5 +193,5 @@ class Plugin(openpaperwork_core.PluginBase):
         while self.core.call_success("fs_exists", dst_url) is not None:
             idx += 1
             dst_url = "{}_{}".format(dst_doc_url, idx)
-        self.core.call_all("fs_rename", src_doc_url, dst_url)
+        self.core.call_success("fs_rename", src_doc_url, dst_url)
         return dst_url
