@@ -2,26 +2,34 @@ import argparse
 import logging
 import sys
 
-import openpaperwork_core
-import openpaperwork_gtk
+import openpaperwork_core  # noqa: E402
+import openpaperwork_gtk  # noqa: E402
 
-import paperwork_backend
+import paperwork_backend  # noqa: E402
 
 # this import must be non-relative due to cx_freeze running this .py
 # as an independant Python script
-from paperwork_gtk import _
+from paperwork_gtk import _  # noqa: E402
 
 
 LOGGER = logging.getLogger(__name__)
 
+REQUIRED_PLUGINS = (
+    'openpaperwork_core.logs.print',
+    # plugin 'uncaught_exceptions' requires a mainloop plugin
+    'openpaperwork_gtk.mainloop.glib',
+)
+
 DEFAULT_GUI_PLUGINS = (
     paperwork_backend.DEFAULT_PLUGINS +
     openpaperwork_gtk.GUI_PLUGINS +
     [
+        'openpaperwork_core.logs.print',
         'openpaperwork_core.spatial.rtree',
         'openpaperwork_gtk.drawer.pillow',
         'openpaperwork_gtk.drawer.scan',
         'openpaperwork_gtk.gesture.autoscrolling',
+        'openpaperwork_gtk.gtk_init',
         'paperwork_backend.docscan.autoselect_scanner',
         'paperwork_backend.guesswork.cropping.calibration',
         'paperwork_gtk.about',
@@ -56,10 +64,11 @@ DEFAULT_GUI_PLUGINS = (
         'paperwork_gtk.actions.page.reset',
         'paperwork_gtk.cmd.import',
         'paperwork_gtk.cmd.install',
-        'paperwork_gtk.docimport',
         'paperwork_gtk.doc_selection',
+        'paperwork_gtk.docimport',
         'paperwork_gtk.drawer.calibration',
         'paperwork_gtk.drawer.frame',
+        'paperwork_gtk.flatpak',
         'paperwork_gtk.gesture.drag_and_drop',
         'paperwork_gtk.gesture.zoom',
         'paperwork_gtk.icon',
@@ -166,6 +175,24 @@ DEFAULT_GUI_PLUGINS = (
 )
 
 
+class DefaultConsole:
+    def print(self, text):
+        print(text)
+
+    def input(self, prompt=""):
+        return input(prompt)
+
+
+def gtk_main(app, options, core):
+    app = core.call_success("gtk_get_app")
+    if app.get_is_remote():
+        LOGGER.info("passing control to main paperwork instance")
+    # if paperwork is already running, focus it, otherwise remain in this
+    # process
+    app.activate()
+    return 0
+
+
 def main_main(in_args):
     # To load the plugins, we need first to load the configuration plugin
     # to get the list of plugins to load.
@@ -173,8 +200,8 @@ def main_main(in_args):
     # enable and configure the plugin logs.print first.
 
     core = openpaperwork_core.Core()
-    # plugin 'uncaught_exceptions' requires a mainloop plugin
-    core.load('openpaperwork_gtk.mainloop.glib')
+    for module_name in REQUIRED_PLUGINS:
+        core.load(module_name)
     for module_name in paperwork_backend.DEFAULT_CONFIG_PLUGINS:
         core.load(module_name)
     core.init()
@@ -187,29 +214,21 @@ def main_main(in_args):
     core.call_all("config_load_plugins", "paperwork-gtk", DEFAULT_GUI_PLUGINS)
 
     if len(in_args) <= 0:
-
-        core.call_all("on_initialized")
-
-        LOGGER.info("Ready")
-        core.call_one("mainloop", halt_on_uncaught_exception=False)
-        LOGGER.info("Quitting")
-        core.call_all("config_save")
-        core.call_all("on_quit")
-
+        core.call_all("gtk_init", gtk_main, core)
     else:
-
+        # we remain in this process and call the plugin requested by the
+        # command line
         parser = argparse.ArgumentParser()
         cmd_parser = parser.add_subparsers(
             help=_('command'), dest='command', required=True
         )
-
         core.call_all("cmd_complete_argparse", cmd_parser)
         args = parser.parse_args(in_args)
+        console = DefaultConsole()
+        core.call_all("cmd_set_console", console)
+        core.call_all("cmd_run", console, args)
 
-        core.call_all("cmd_set_interactive", True)
-
-        core.call_all("cmd_run", args)
-        core.call_all("on_quit")
+    return 0
 
 
 def main():
@@ -217,5 +236,5 @@ def main():
 
 
 if __name__ == "__main__":
-    # Do not remove. Cx_freeze goes throught here
+    # Do not remove. Cx_freeze goes through here
     main()
