@@ -9,14 +9,13 @@ except (ImportError, ValueError):
     GLIB_AVAILABLE = False
 
 try:
-    import gi
-    gi.require_version('Gtk', '3.0')
-    from gi.repository import Gtk
+    from . import doc_box
     GTK_AVAILABLE = True
 except (ImportError, ValueError):
     GTK_AVAILABLE = False
 
 import openpaperwork_core
+import openpaperwork_core.deps
 import openpaperwork_gtk.deps
 
 from ... import _
@@ -31,6 +30,8 @@ NB_DOCS_PER_PAGE = 50
 
 
 class Plugin(openpaperwork_core.PluginBase):
+    PRIORITY = -100
+
     def __init__(self):
         super().__init__()
         self.widget_tree = None
@@ -40,10 +41,11 @@ class Plugin(openpaperwork_core.PluginBase):
         self._scrollbar_last_value = -1
 
         self.docs = []
-        self.doc_visibles = 0
+        self.row_visibles = []
+        self.doc_loaded = 0
         self.row_to_doc = {}
         self.docid_to_row = {}
-        self.docid_to_widget_tree = {}
+        self.min_doc_box_height = 1
 
         self.last_date = datetime.datetime(year=1, month=1, day=1)
 
@@ -152,6 +154,7 @@ class Plugin(openpaperwork_core.PluginBase):
         self.doclist.connect("row-activated", self._on_row_activated)
         self.doclist.connect("drag-motion", self._on_drag_motion)
         self.doclist.connect("drag-leave", self._on_drag_leave)
+        self.doclist.connect("size-allocate", self._notify_box_visibility)
 
         self.widget_tree.get_object("doclist_new_doc").connect(
             "clicked", self._on_new_doc
@@ -174,12 +177,19 @@ class Plugin(openpaperwork_core.PluginBase):
             "mainloop_schedule", self.core.call_all, "on_doclist_initialized"
         )
 
+        self.on_mainwindow_fold_change()
+
     def chkdeps(self, out: dict):
         if not GLIB_AVAILABLE:
-            out['glib'].update(openpaperwork_gtk.deps.GLIB)
+            out['glib'].update(openpaperwork_core.deps.GLIB)
         if not GTK_AVAILABLE:
             out['gtk'].update(openpaperwork_gtk.deps.GTK)
 
+    def on_gtk_initialized(self):
+        self.core.call_all(
+            "mainwindow_show", side="left", name="doclist"
+        )
+
     def doclist_add(self, widget, vposition):
         body = self.widget_tree.get_object("doclist_body")
         body.add(widget)
@@ -193,26 +203,25 @@ class Plugin(openpaperwork_core.PluginBase):
         self.core.call_all("doc_open", *new_doc)
         self.doclist_show(self.docs, show_new=True)
 
-    def _doclist_clear(self):
-        start = time.time()
+    def doclist_clear(self):
+        self.min_doc_box_height = 1
+        self.core.call_all("on_doc_list_clear")
 
+        start = time.time()
         for child in self.doclist.get_children():
             self.doclist.remove(child)
-
         stop = time.time()
 
         LOGGER.info(
             "%d documents cleared in %dms",
-            self.doc_visibles, (stop - start) * 1000
+            self.doc_loaded, (stop - start) * 1000
         )
 
-    def doclist_clear(self):
-        self._doclist_clear()
         self.last_date = datetime.datetime(year=1, month=1, day=1)
-        self.doc_visibles = 0
+        self.doc_loaded = 0
         self.row_to_doc = {}
         self.docid_to_row = {}
-        self.docid_to_widget_tree = {}
+        self.row_visibles = []
 
     def _add_date_box(self, name, txt):
         widget_tree = self.core.call_success(
@@ -230,101 +239,88 @@ class Plugin(openpaperwork_core.PluginBase):
             self.core.call_all("doc_selection_remove", doc_id, doc_url)
 
     def _add_doc_box(self, doc_id, doc_url, box="doc_box.glade", new=False):
-        widget_tree = self.core.call_success(
-            "gtk_load_widget_tree",
-            "paperwork_gtk.mainwindow.doclist", box
-        )
-
-        doc_box = widget_tree.get_object("doc_box")
+        row = doc_box.DocBox()
 
         flowlayout = self.core.call_success(
             "gtk_widget_flowlayout_new", spacing=(3, 3)
         )
         flowlayout.set_visible(True)
-        doc_box.pack_start(flowlayout, expand=True, fill=True, padding=0)
-        doc_box.reorder_child(flowlayout, 1)
+        row.layout_bin.add(flowlayout)
 
         self.core.call_all(
-            "on_doc_box_creation", doc_id, widget_tree, flowlayout
+            "on_doc_box_creation", doc_id, row, flowlayout
         )
 
-        doc_actions = widget_tree.get_object("doc_actions")
+        doc_actions = row.main_actions
         if new:
             doc_actions.set_visible(False)
         else:
-            widget_tree.get_object("doc_actions_menu").set_menu_model(
-                self.doc_actions
-            )
+            row.action_menu.set_menu_model(self.doc_actions)
 
         for action in self.main_actions:
-            button = self.core.call_success(
-                "gtk_load_widget_tree",
-                "paperwork_gtk.mainwindow.doclist", "main_action.glade"
+            action = doc_box.DocMainAction(
+                action['txt'], action['icon_name'], action['callback']
             )
-            button.get_object("doc_main_action_image").set_from_icon_name(
-                action['icon_name'], Gtk.IconSize.MENU
-            )
-            button.get_object("doc_main_action").set_tooltip_text(
-                action['txt']
-            )
-            button.get_object("doc_main_action").connect(
-                "clicked", lambda _: action['callback']()
-            )
-            widget_tree.get_object("doc_actions").pack_start(
-                button.get_object("doc_main_action"),
-                expand=True, fill=True, padding=0
+            row.main_actions.pack_start(
+                action, expand=True, fill=True, padding=0
             )
 
-        widget_tree.get_object("doc_box_selector").set_visible(
+        row.selector.set_visible(
             self.selection_multiple
         )
-        widget_tree.get_object("doc_box_selector").set_active(
+        row.selector.set_active(
             self.core.call_success("doc_selection_in", doc_id, doc_url)
             is not None
         )
-        widget_tree.get_object("doc_box_selector").connect(
+        row.selector.connect(
             "toggled", self._toggle_doc, doc_id, doc_url
         )
 
-        row = widget_tree.get_object("doc_listbox")
+        row.connect("size-allocate", self._on_doc_box_size_allocate)
+
         self.row_to_doc[row] = (doc_id, doc_url)
         self.docid_to_row[doc_id] = row
-        self.docid_to_widget_tree[doc_id] = widget_tree
         self.doclist.insert(row, -1)
 
     def doclist_extend(self, nb_docs):
         start = time.time()
+        # FIXME: show month only when there are too many documents. How many?
+        show_month = True
 
         docs = self.docs[
-            self.doc_visibles:self.doc_visibles + nb_docs
+            self.doc_loaded:self.doc_loaded + nb_docs
         ]
         LOGGER.info(
             "Adding %d documents to the document list (%d-%d)",
-            len(docs), self.doc_visibles, self.doc_visibles + nb_docs
+            len(docs), self.doc_loaded, self.doc_loaded + nb_docs
         )
 
         for (doc_id, doc_url) in docs:
             doc_date = self.core.call_success("doc_get_date_by_id", doc_id)
 
             if doc_date is not None:
-                if doc_date.year != self.last_date.year:
-                    doc_year = self.core.call_success(
-                        "i18n_date_long_year", doc_date
-                    )
-                    self._add_date_box("year_box.glade", doc_year)
+                if not show_month:
+                    if doc_date.year != self.last_date.year:
+                        doc_year = self.core.call_success(
+                            "i18n_date_long_year", doc_date
+                        )
+                        self._add_date_box("year_box.glade", doc_year)
 
-                if (doc_date.year != self.last_date.year or
-                        doc_date.month != self.last_date.month):
-                    doc_month = self.core.call_success(
-                        "i18n_date_long_month", doc_date
-                    )
-                    self._add_date_box("month_box.glade", doc_month)
+                else:
+                    if (doc_date.year != self.last_date.year or
+                            doc_date.month != self.last_date.month):
+                        doc_year = self.core.call_success(
+                            "i18n_date_long_year", doc_date
+                        ) + " / " + self.core.call_success(
+                            "i18n_date_long_month", doc_date
+                        )
+                        self._add_date_box("year_box.glade", doc_year)
 
                 self.last_date = doc_date
 
             self._add_doc_box(doc_id, doc_url)
 
-        self.doc_visibles = min(len(self.docs), self.doc_visibles + nb_docs)
+        self.doc_loaded = min(len(self.docs), self.doc_loaded + nb_docs)
 
         stop = time.time()
 
@@ -348,16 +344,8 @@ class Plugin(openpaperwork_core.PluginBase):
         self.doclist_extend(NB_DOCS_PER_PAGE)
         self._reselect_current_doc(scroll=scroll)
 
-    def on_search_start(self, query):
-        spinner = self.widget_tree.get_object("doclist_spinner")
-        spinner.set_visible(True)
-        spinner.start()
-
     def on_search_results(self, query, docs):
         self.doclist_show(docs, show_new=(query == ""))
-        spinner = self.widget_tree.get_object("doclist_spinner")
-        spinner.set_visible(False)
-        spinner.stop()
 
     def doc_close(self):
         self.active_doc = (None, None)
@@ -385,19 +373,19 @@ class Plugin(openpaperwork_core.PluginBase):
             return
 
         row = self._show_doc_id(doc_id)
-        assert(row is not None)
+        assert row is not None
         self.doclist.select_row(row)
 
         if (self.previous_doc[0] is not None and
-                self.previous_doc[0] in self.docid_to_widget_tree):
-            widget_tree = self.docid_to_widget_tree[self.previous_doc[0]]
-            widget_tree.get_object("doc_actions").set_visible(False)
+                self.previous_doc[0] in self.docid_to_row):
+            row = self.docid_to_row[self.previous_doc[0]]
+            row.main_actions.set_visible(False)
 
         if ((not self.selection_multiple) and
                 self.core.call_success("is_doc", doc_url) is not None and
-                doc_id in self.docid_to_widget_tree):
-            widget_tree = self.docid_to_widget_tree[doc_id]
-            widget_tree.get_object("doc_actions").set_visible(True)
+                doc_id in self.docid_to_row):
+            row = self.docid_to_row[doc_id]
+            row.main_actions.set_visible(True)
 
         self.previous_doc = self.active_doc
 
@@ -418,9 +406,46 @@ class Plugin(openpaperwork_core.PluginBase):
 
         handler_id = row.connect("size-allocate", scroll_to_row)
 
+    def _on_doc_box_size_allocate(self, box, allocation):
+        h = allocation.height
+        if h <= 0:
+            return
+        if self.min_doc_box_height <= 1:
+            self.min_doc_box_height = h
+        else:
+            self.min_doc_box_height = min(self.min_doc_box_height, h)
+        self._notify_box_visibility()
+
+    def _notify_box_visibility(self, *args, **kwargs):
+        lower = int(self.vadj.get_value())
+        upper = int(lower + self.vadj.get_page_size())
+
+        row_visibles = []
+        prow = None
+        for y in range(lower, upper, self.min_doc_box_height):
+            row = self.doclist.get_row_at_y(y)
+            if row is None or prow is row:
+                continue
+            prow = row
+            row_visibles.append(row)
+
+        if self.row_visibles == row_visibles:
+            return
+        self.row_visibles = row_visibles
+
+        self.core.call_all("on_doc_list_visibility_changed")
+        for row in row_visibles:
+            doc = self.row_to_doc.get(row)
+            if doc is None:
+                continue
+            (doc_id, doc_url) = doc
+            layout = row.layout_bin.get_child()
+            self.core.call_all("on_doc_box_visible", doc_id, row, layout)
+
     def _on_scrollbar_value_changed(self, vadj):
         lower = vadj.get_lower()
         upper = vadj.get_upper() - lower
+
         value = (
             vadj.get_value() +
             vadj.get_page_size() -
@@ -428,14 +453,17 @@ class Plugin(openpaperwork_core.PluginBase):
         ) / upper
 
         if value < 0.95:
+            self._notify_box_visibility()
             return
 
         if self._scrollbar_last_value == vadj.get_value():
             # Previous extend call hasn't been taken into account yet
+            self._notify_box_visibility()
             return
 
         self.doclist_extend(NB_DOCS_PER_PAGE)
         self._scrollbar_last_value = vadj.get_value()
+        self._notify_box_visibility()
 
     def _doc_open(self, doc_id, doc_url):
         if self.active_doc[0] == doc_id:
@@ -530,9 +558,9 @@ class Plugin(openpaperwork_core.PluginBase):
 
         if self.active_doc[0] is None:
             return
-        widget_tree = self.docid_to_widget_tree[self.active_doc[0]]
+        row = self.docid_to_row[self.active_doc[0]]
         self.core.call_success(
-            "screenshot_snap_widget", widget_tree.get_object("doc_actions"),
+            "screenshot_snap_widget", row.main_actions,
             self.core.call_success(
                 "fs_join", out_dir, "doc_properties_button.png"
             ),
@@ -540,8 +568,8 @@ class Plugin(openpaperwork_core.PluginBase):
         )
 
     def _set_all_selector_visibility(self, visible):
-        for widget_tree in self.docid_to_widget_tree.values():
-            checkbox = widget_tree.get_object("doc_box_selector")
+        for row in self.docid_to_row.values():
+            checkbox = row.selector
             checkbox.set_visible(visible)
 
     def gtk_switch_to_doc_selection_single(self):
@@ -567,8 +595,8 @@ class Plugin(openpaperwork_core.PluginBase):
 
         self._toggling = True
         try:
-            for widget_tree in self.docid_to_widget_tree.values():
-                checkbox = widget_tree.get_object("doc_box_selector")
+            for row in self.docid_to_row.values():
+                checkbox = row.selector
                 checkbox.set_active(False)
         finally:
             self._toggling = False
@@ -580,10 +608,10 @@ class Plugin(openpaperwork_core.PluginBase):
             return
         self._toggling = True
         try:
-            widget_tree = self.docid_to_widget_tree.get(doc_id, None)
-            if widget_tree is None:
+            row = self.docid_to_row.get(doc_id, None)
+            if row is None:
                 return
-            checkbox = widget_tree.get_object("doc_box_selector")
+            checkbox = row.selector
             if not checkbox.get_active():
                 checkbox.set_active(True)
         finally:
@@ -596,10 +624,10 @@ class Plugin(openpaperwork_core.PluginBase):
             return
         self._toggling = True
         try:
-            widget_tree = self.docid_to_widget_tree.get(doc_id, None)
-            if widget_tree is None:
+            row = self.docid_to_row.get(doc_id, None)
+            if row is None:
                 return
-            checkbox = widget_tree.get_object("doc_box_selector")
+            checkbox = row.selector
             if checkbox.get_active():
                 checkbox.set_active(False)
         finally:
@@ -631,16 +659,16 @@ class Plugin(openpaperwork_core.PluginBase):
         if self.active_doc[0] is None:
             return
         doc_id = self.active_doc[0]
-        widget_tree = self.docid_to_widget_tree[doc_id]
-        button = widget_tree.get_object("doc_actions_menu")
+        row = self.docid_to_row[doc_id]
+        button = row.action_menu
         button.clicked()
 
     def screenshot_snap_doc_action_menu(self, out_file):
         if self.active_doc[0] is None:
             return
         doc_id = self.active_doc[0]
-        widget_tree = self.docid_to_widget_tree[doc_id]
-        button = widget_tree.get_object("doc_actions_menu")
+        row = self.docid_to_row[doc_id]
+        button = row.main_actions
 
         self.core.call_success(
             "screenshot_snap_widget",
@@ -648,3 +676,9 @@ class Plugin(openpaperwork_core.PluginBase):
             out_file,
             margins=(100, 50, 100, 200)
         )
+
+    def on_mainwindow_fold_change(self):
+        folded = self.core.call_success("mainwindow_get_folded")
+        self.widget_tree.get_object("doclist_header").set_show_close_button(
+            folded
+        )
