@@ -11,6 +11,11 @@ except (ImportError, ValueError):
 
     # workaround so chkdeps can still be called
     class GObject(object):
+        TYPE_BOOLEAN = 0
+
+        class SignalFlags(object):
+            RUN_LAST = 0
+
         class GObject(object):
             pass
 
@@ -166,7 +171,7 @@ class Plugin(openpaperwork_core.PluginBase):
             {
                 'interface': 'gtk_drawer_scan',
                 'defaults': [
-                    'paperwork_backend.drawer.scan',
+                    'openpaperwork_gtk.drawer.scan',
                     'paperwork_gtk.drawer.calibration',
                 ],
             },
@@ -176,15 +181,15 @@ class Plugin(openpaperwork_core.PluginBase):
         self.doc_id = doc_id
         self.doc_url = doc_url
 
-        nb_pages = self.core.call_success(
-            "doc_get_nb_pages_by_url", doc_url
-        )
+        nb_pages = self.core.call_success("doc_get_nb_pages_by_url", doc_url)
         if nb_pages is None:
             nb_pages = 0
         scan_id = self.core.call_success("scan2doc_doc_id_to_scan_id", doc_id)
         self.scan = Scan(self.core, self, doc_id, nb_pages, scan_id)
-        out.append(self.scan)
-        if scan_id is not None:
+        if scan_id is None:
+            out.append((False, self.scan))
+        else:
+            out.append((True, self.scan))
             self.scan.start()
 
     def on_scan2doc_start(self, scan_id, doc_id, doc_url):
@@ -200,6 +205,7 @@ class Plugin(openpaperwork_core.PluginBase):
         if scan_id != self.scan.scan_id:
             return
         self.scan.start()
+        self.core.call_all("docview_show_page_viewer", self.scan)
 
     def on_scan_page_start(self, scan_id, page_nb, scan_params):
         self.scan_sizes[scan_id] = (
@@ -237,7 +243,7 @@ class Plugin(openpaperwork_core.PluginBase):
         nb_pages = self.core.call_success(
             "doc_get_nb_pages_by_url", self.doc_url
         )
-        assert(nb_pages is not None)
+        assert nb_pages is not None
 
         LOGGER.info("Displaying new page %d", nb_pages - 1)
         self.core.call_all("doc_goto_page", nb_pages - 1)
@@ -248,6 +254,7 @@ class Plugin(openpaperwork_core.PluginBase):
             return
         if scan_id != self.scan.scan_id:
             return
+        self.core.call_all("docview_hide_page_viewer", self.scan)
         self.scan.stop()
 
     def on_scan2doc_end(self, scan_id, doc_id, doc_url):
